@@ -1,6 +1,10 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 from pathlib import Path
 from shutil import which
@@ -44,21 +48,25 @@ def main():
         sim_modules = [module for module in all_modules if len(module.get_ip_core_files()) == 0]
 
     if args.vivado_skip:
-        ip_core_vivado_project_sources_directory = None
+        ip_core_vivado_project_directory = None
     else:
         add_simlib(vunit_proj, args.temp_dir, args.simlib_compile)
 
         # Generate IP core simulation files. Will be used for the vhdl_ls config,
         # even if they are not added to the simulation project.
-        ip_core_compile_order_file, ip_core_vivado_project_sources_directory \
-            = generate_ip_core_files(all_modules, args.temp_dir, args.ip_compile)
+        (
+            ip_core_compile_order_file,
+            ip_core_vivado_project_directory,
+        ) = generate_ip_core_files(all_modules, args.temp_dir, args.ip_compile)
         if has_commercial_simulator:
             add_from_compile_order_file(vunit_proj, ip_core_compile_order_file)
 
-    create_vhdl_ls_configuration(PATH_TO_TSFPGA,
-                                 vunit_proj,
-                                 all_modules,
-                                 ip_core_vivado_project_sources_directory)
+    create_vhdl_ls_configuration(
+        output_path=PATH_TO_TSFPGA,
+        vunit_proj=vunit_proj,
+        modules=all_modules,
+        ip_core_vivado_project_directory=ip_core_vivado_project_directory,
+    )
 
     for module in sim_modules:
         vunit_library = vunit_proj.add_library(module.library_name)
@@ -67,35 +75,34 @@ def main():
                 vunit_library.add_source_file(hdl_file.path)
             else:
                 assert False, f"Can not handle this file: {hdl_file}"
-        module.setup_simulations(vunit_proj)
+        module.setup_vunit(vunit_proj)
 
     if vunit_proj.simulator_supports_coverage():
         vunit_proj.set_compile_option("enable_coverage", True)
         vunit_proj.set_sim_option("enable_coverage", True)
+
         vunit_proj.main(post_run=merge_ghdl_coverage)
     else:
-        # Don't enable psl when running with coverage, as the gcc
-        # backend seems to have some issues with psl
-        vunit_proj.set_compile_option("ghdl.a_flags", ["-fpsl"])
-
         vunit_proj.main()
 
 
 def arguments():
     cli = VUnitCLI()
-    cli.parser.add_argument("--temp-dir",
-                            type=Path,
-                            default=TSFPGA_EXAMPLES_TEMP_DIR,
-                            help="where to place files needed for simulation flow")
-    cli.parser.add_argument("--vivado-skip",
-                            action="store_true",
-                            help="skip all steps that require Vivado")
-    cli.parser.add_argument("--ip-compile",
-                            action="store_true",
-                            help="force (re)compile of IP cores")
-    cli.parser.add_argument("--simlib-compile",
-                            action="store_true",
-                            help="force (re)compile of Vivado simlib")
+    cli.parser.add_argument(
+        "--temp-dir",
+        type=Path,
+        default=TSFPGA_EXAMPLES_TEMP_DIR,
+        help="where to place files needed for simulation flow",
+    )
+    cli.parser.add_argument(
+        "--vivado-skip", action="store_true", help="skip all steps that require Vivado"
+    )
+    cli.parser.add_argument(
+        "--ip-compile", action="store_true", help="force (re)compile of IP cores"
+    )
+    cli.parser.add_argument(
+        "--simlib-compile", action="store_true", help="force (re)compile of Vivado simlib"
+    )
 
     args = cli.parse_args()
     args.output_path = args.temp_dir / "vunit_out"
@@ -122,13 +129,16 @@ def generate_ip_core_files(modules, temp_dir, force_generate):
     if vivado_project_created:
         # If the IP core Vivado project has been (re)created we need to create
         # a new compile order file
-        create_compile_order_file(vivado_ip_cores.vivado_project_file,
-                                  vivado_ip_cores.compile_order_file)
+        create_compile_order_file(
+            vivado_ip_cores.vivado_project_file, vivado_ip_cores.compile_order_file
+        )
 
-    return vivado_ip_cores.compile_order_file, vivado_ip_cores.vivado_project_sources_directory
+    return vivado_ip_cores.compile_order_file, vivado_ip_cores.project_directory
 
 
-def create_vhdl_ls_configuration(output_path, vunit_proj, modules, ip_core_vivado_project_sources_directory):
+def create_vhdl_ls_configuration(
+    output_path, vunit_proj, modules, ip_core_vivado_project_directory
+):
     """
     Create config for vhdl_ls. Granted this might no be the "correct" place for this functionality.
     But since the call is somewhat quick (~10 ms), and simulate.py is run "often" it seems an
@@ -140,7 +150,8 @@ def create_vhdl_ls_configuration(output_path, vunit_proj, modules, ip_core_vivad
         modules=modules,
         vunit_proj=vunit_proj,
         vivado_location=vivado_location,
-        ip_core_vivado_project_sources_directory=ip_core_vivado_project_sources_directory)
+        ip_core_vivado_project_directory=ip_core_vivado_project_directory,
+    )
 
 
 def merge_ghdl_coverage(results):
@@ -155,19 +166,20 @@ def merge_ghdl_coverage(results):
 
     report_html_path = temp_dir / "vhdl_coverage_html"
     create_directory(report_html_path, empty=True)
-    gcovr_cmd = ["gcovr",
-                 "--xml",
-                 temp_dir / "vhdl_coverage.xml",
-                 "--xml-pretty",
-                 "--html",
-                 "--html-details",
-                 report_html_path / "index.html",
-                 "--html-title",
-                 "tsfpga VHDL coverage",
-                 "--exclude-unreachable-branches",
-                 "--exclude-throw-branches",
-                 merged_coverage_output,
-                 ]
+    gcovr_cmd = [
+        "gcovr",
+        "--xml",
+        temp_dir / "vhdl_coverage.xml",
+        "--xml-pretty",
+        "--html",
+        "--html-details",
+        report_html_path / "index.html",
+        "--html-title",
+        "tsfpga VHDL coverage",
+        "--exclude-unreachable-branches",
+        "--exclude-throw-branches",
+        merged_coverage_output,
+    ]
     subprocess.call(gcovr_cmd)
 
     print(f"Coverage HTML report saved in {report_html_path}")
