@@ -1,6 +1,10 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 """
 A set of reusable functions for working with HDL projects.
@@ -23,6 +27,10 @@ TSFPGA_EXAMPLES = REPO_ROOT / "examples"
 TSFPGA_EXAMPLE_MODULES = TSFPGA_EXAMPLES / "modules"
 TSFPGA_EXAMPLE_MODULES_WITH_IP = TSFPGA_EXAMPLES / "modules_with_ip"
 
-ALL_TSFPGA_MODULES_FOLDERS = [TSFPGA_MODULES, TSFPGA_EXAMPLE_MODULES, TSFPGA_EXAMPLE_MODULES_WITH_IP]
+ALL_TSFPGA_MODULES_FOLDERS = [
+    TSFPGA_MODULES,
+    TSFPGA_EXAMPLE_MODULES,
+    TSFPGA_EXAMPLE_MODULES_WITH_IP,
+]
 
-__version__ = "4.0.1"
+__version__ = "5.0.0"
