@@ -1,13 +1,16 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
-
-import copy
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 from tsfpga.constraint import Constraint
 from tsfpga.hdl_file import HdlFile
-from tsfpga.register_list import from_toml
 from tsfpga.system_utils import load_python_module
+from tsfpga.registers.parser import from_toml
+from tsfpga.module_list import ModuleList
 
 
 class BaseModule:
@@ -20,7 +23,7 @@ class BaseModule:
 
     def __init__(self, path, library_name, default_registers=None):
         """
-        Args:
+        Arguments:
             path (`pathlib.Path`): Path to the module folder.
             library_name (str): VHDL library name.
             default_registers (list(Register)): Default registers.
@@ -29,27 +32,46 @@ class BaseModule:
         self.name = path.name
         self.library_name = library_name
 
-        # Note: Likely mutable object, need to create deep copy before using.
         self._default_registers = default_registers
         self._registers = None
 
     @staticmethod
-    def _get_file_list(folders, file_endings):
+    def _get_file_list(folders, file_endings, files_include=None, files_avoid=None):
         """
         Returns a list of files given a list of folders.
+
+        Arguments:
+            folders (pathlib.Path): The forlders to search.
+            file_endings (tuple(str)): File endings to include.
+            files_include (set(pathlib.Path)): Optionally filter to only include these files.
+            files_avoid (set(pathlib.Path)): Optionally filter to discard these files.
         """
         files = []
         for folder in folders:
             for file in folder.glob("*"):
-                if file.is_file() and file.name.lower().endswith(file_endings):
+                # pylint: disable=too-many-boolean-expressions
+                if (
+                    file.is_file()
+                    and file.name.lower().endswith(file_endings)
+                    and (files_include is None or file in files_include)
+                    and (files_avoid is None or file not in files_avoid)
+                ):
                     files.append(file)
         return files
 
-    def _get_hdl_file_list(self, folders):
+    def _get_hdl_file_list(self, folders, files_include, files_avoid):
         """
         Return a list of HDL file objects.
         """
-        return [HdlFile(filename) for filename in self._get_file_list(folders, HdlFile.file_endings)]
+        return [
+            HdlFile(file_path)
+            for file_path in self._get_file_list(
+                folders=folders,
+                file_endings=HdlFile.file_endings,
+                files_include=files_include,
+                files_avoid=files_avoid,
+            )
+        ]
 
     @property
     def registers(self):
@@ -62,16 +84,10 @@ class BaseModule:
 
         toml_file = self.path / (f"regs_{self.name}.toml")
         if toml_file.exists():
-            self._registers = from_toml(self.name, toml_file, copy.deepcopy(self._default_registers))
+            self._registers = from_toml(self.name, toml_file, self._default_registers)
             self.registers_hook()
             return self._registers
 
-        for deprecated_json_file in [self.path / f"{self.name}_regs.json", self.path / f"regs_{self.name}.json"]:
-            if deprecated_json_file.exists():
-                message = f"DEPRECATED: Using deprecated json file name: {deprecated_json_file}"
-                message += f"\nDEPRECATED: Use TOML file instead: {toml_file}"
-                raise ValueError(message)
-
         return None
 
     def registers_hook(self):
@@ -83,7 +99,8 @@ class BaseModule:
         Override this method and implement the desired behavior in a child class.
 
         .. Note::
-            This default method does nothing and can be safely overridden.
+            This default method does nothing. Shall be overridden by modules that utilize
+            this mechanism.
         """
 
     def create_regs_vhdl_package(self):
@@ -93,8 +110,21 @@ class BaseModule:
         if self.registers is not None:
             self.registers.create_vhdl_package(self.path)
 
-    def get_synthesis_files(self):
+    def get_synthesis_files(self, files_include=None, files_avoid=None):
         """
+        Get a list of files that shall be included in a synthesis project.
+
+        The ``files_include`` and ``files_avoid`` arguments can be used to filter what files are
+        included.
+        This can be useful in many situations, e.g. when encrypted files of files that include an
+        IP core shall be avoided.
+        It is recommended to overload this function in a child class in your ``module_*.py``,
+        and call this super method with the arguments supplied.
+
+        Arguments:
+            files_include (set(`pathlib.Path`)): Optionally filter to only include these files.
+            files_avoid (set(`pathlib.Path`)): Optionally filter to discard these files.
+
         Return:
             list(:class:`.HdlFile`): Files that should be included in a synthesis project.
         """
@@ -107,37 +137,67 @@ class BaseModule:
             self.path / "hdl" / "rtl",
             self.path / "hdl" / "package",
         ]
-        return self._get_hdl_file_list(folders)
+        return self._get_hdl_file_list(
+            folders=folders, files_include=files_include, files_avoid=files_avoid
+        )
 
-    def get_simulation_files(self, include_tests=True):
+    def get_simulation_files(self, include_tests=True, files_include=None, files_avoid=None):
         """
-        Args:
-            include_tests (bool): When False the test folder is not included.
-                The use case of include_tests is when testing a primary module
-                that depends on other secondary modules, we may want to compile
-                the simulation files (``sim`` folder) of the secondary modules but not their
-                test files (``test`` folder).
+        See :meth:`.get_synthesis_files` for insctructions on how to use ``files_include``
+        and ``files_avoid``.
+
+        Arguments:
+            include_tests (bool): When ``False`` the ``test`` folder is not included.
+                The use case is when testing a primary module that depends on other
+                secondary modules.
+                In this case we may want to compile the simulation files (``sim`` folder) of the
+                secondary modules but not their test files (``test`` folder).
 
                 .. Note::
-                    `test` files are considered private to the module and should never be used by
-                    other modules.
+                    The ``test`` files are considered private to the module and should never be used
+                    by other modules.
+            files_include (set(`pathlib.Path`)): Optionally filter to only include these files.
+            files_avoid (set(`pathlib.Path`)): Optionally filter to discard these files.
 
         Return:
             list(:class:`.HdlFile`): Files that should be included in a simulation project.
         """
-        self.create_regs_vhdl_package()
-
         test_folders = [
             self.path / "sim",
         ]
 
         if include_tests:
-            test_folders += [self.path / "rtl" / "tb",
-                             self.path / "test"]
+            test_folders += [self.path / "rtl" / "tb", self.path / "test"]
 
-        return self.get_synthesis_files() + self._get_hdl_file_list(test_folders)
+        synthesis_files = self.get_synthesis_files(
+            files_include=files_include, files_avoid=files_avoid
+        )
+        test_files = self._get_hdl_file_list(
+            test_folders, files_include=files_include, files_avoid=files_avoid
+        )
 
-    def setup_simulations(self, vunit_proj, **kwargs):
+        return synthesis_files + test_files
+
+    def get_formal_files(self, files_include=None, files_avoid=None):
+        """
+        Returns the files to be used for formal verification.
+        By default these are the same that are used by synthesis
+        (by calling :meth:`get_synthesis_files`).
+        Overload this method to select files manually.
+
+        See :meth:`.get_synthesis_files` for insctructions on how to use ``files_include``
+        and ``files_avoid``.
+
+        Arguments:
+            files_include (set(`pathlib.Path`)): Optionally filter to only include these files.
+            files_avoid (set(`pathlib.Path`)): Optionally filter to discard these files.
+
+        Return:
+            list(:class:`.HdlFile`): Files that should be included in a formal verification project.
+        """
+        return self.get_synthesis_files(files_include=files_include, files_avoid=files_avoid)
+
+    def setup_vunit(self, vunit_proj, **kwargs):
         """
         Setup local configuration of this module's test benches.
 
@@ -145,13 +205,20 @@ class BaseModule:
             This default method does nothing. Should be overridden by modules that have
             any test benches that operate via generics.
 
-        Args:
+        Arguments:
             vunit_proj: The VUnit project that is used to run simulation.
             kwargs: Use this to pass an arbitrary list of arguments from your ``simulate.py``
                 to the module where you set up your tests. This could be, e.g., data dimensions,
                 location of test files, etc.
         """
 
+    # pylint: disable=unused-argument,no-self-use
+    def setup_simulations(self, vunit_proj, **kwargs):
+        print(
+            "DEPRECATED: Calling deprecated BaseModule.setup_simulations(). "
+            "Use BaseModule.setup_vunit() instead."
+        )
+
     def setup_formal(self, formal_proj, **kwargs):
         """
         Setup this module's formal tests.
@@ -159,8 +226,34 @@ class BaseModule:
         .. Note::
             This default method does nothing. Should be overridden by modules that
             utilize formal verification.
+
+        Arguments:
+            formal_proj: The formal project that is being used.
+            kwargs: Use this to pass an arbitrary list of arguments from your ``formal.py``
+                to the module where you set up your tests. This could be, e.g., data dimensions,
+                location of test files, etc.
         """
 
+    def pre_build(self, project, **kwargs):
+        """
+        This method hook will be called before an FPGA build is run. A typical use case for this
+        mechanism is to set a register constant or default value based on the generics that
+        are passed to the project. Could also be used to, e.g., generate BRAM init files
+        based on project information, etc.
+
+        .. Note::
+            This default method does nothing. Should be overridden by modules that
+            utilize this mechanism.
+
+        Arguments:
+            project (.VivadoProject): The project that is being built.
+            kwargs: All parameters that are sent to the :meth:`.VivadoProject.build` method.
+
+        Return:
+            bool: True if everything went well.
+        """
+        return True
+
     def get_ip_core_files(self):
         """
         Return:
@@ -169,7 +262,7 @@ class BaseModule:
         folders = [
             self.path / "ip_cores",
         ]
-        file_endings = ("tcl")
+        file_endings = "tcl"
         return self._get_file_list(folders, file_endings)
 
     def get_scoped_constraints(self):
@@ -190,9 +283,12 @@ class BaseModule:
         if constraint_files:
             synthesis_files = self.get_synthesis_files()
             for constraint_file in constraint_files:
-                # Scoped constraints often depend on clocks having been created by another constraint
-                # file before they can work. Set processing order to "late" to make this more probable.
-                constraint = Constraint(constraint_file, scoped_constraint=True, processing_order="late")
+                # Scoped constraints often depend on clocks having been created by another
+                # constraint file before they can work. Set processing order to "late" to make
+                # this more probable.
+                constraint = Constraint(
+                    constraint_file, scoped_constraint=True, processing_order="late"
+                )
                 constraint.validate_scoped_entity(synthesis_files)
                 constraints.append(constraint)
         return constraints
@@ -202,7 +298,8 @@ class BaseModule:
         Get FPGA build projects defined by this module.
 
         .. Note::
-            Should be overridden by modules that set up build projects.
+            This default method does nothing. Should be overridden by modules that set up
+            build projects.
 
         Return:
             list(:class:`.VivadoProject`): FPGA build projects.
@@ -212,7 +309,7 @@ class BaseModule:
     @staticmethod
     def test_case_name(name=None, generics=None):
         """
-        Costruct a string suitable for naming test cases.
+        Construct a string suitable for naming test cases.
 
         Example result: MyName.GenericA_ValueA.GenericB_ValueB
         """
@@ -230,12 +327,12 @@ class BaseModule:
 
         return test_case_name
 
-    def add_config(self, test, name=None, generics=None):
+    def add_vunit_config(self, test, name=None, generics=None, pre_config=None, post_check=None):
         """
         Add config for VUnit test case. Wrapper that sets a suitable name.
         """
         name = self.test_case_name(name, generics)
-        test.add_config(name=name, generics=generics)
+        test.add_config(name=name, generics=generics, pre_config=pre_config, post_check=post_check)
 
     def __str__(self):
         return f"{self.name}:{self.path}"
@@ -257,15 +354,17 @@ def get_module_object(path, name, library_name_has_lib_suffix, default_registers
     return BaseModule(path, library_name, default_registers)
 
 
-def get_modules(modules_folders,
-                names_include=None,
-                names_avoid=None,
-                library_name_has_lib_suffix=False,
-                default_registers=None):
+def get_modules(
+    modules_folders,
+    names_include=None,
+    names_avoid=None,
+    library_name_has_lib_suffix=False,
+    default_registers=None,
+):
     """
     Get a list of Module objects based on the source code folders.
 
-    Args:
+    Arguments:
         modules_folders (list(`pathlib.Path`)): A list of paths where your modules are located.
         names_include (list(str)): If specified, only modules with these names will be included.
         names_avoid (list(str)): If specified, modules with these names will be discarded.
@@ -274,24 +373,20 @@ def get_modules(modules_folders,
         default_registers (list(Register)): Default registers.
 
     Return:
-        List of module objects (:class:`BaseModule` or child classes thereof) created from
-        the specified folders.
+        :class:`.ModuleList`: List of module objects (:class:`BaseModule` or child classes thereof)
+        created from the specified folders.
     """
-    modules = []
+    modules = ModuleList()
 
     for module_folder in iterate_module_folders(modules_folders):
         module_name = module_folder.name
-        if (names_include is None or module_name in names_include) \
-                and (names_avoid is None or module_name not in names_avoid):
-            modules.append(get_module_object(module_folder,
-                                             module_name,
-                                             library_name_has_lib_suffix,
-                                             default_registers))
-
-            deprecated_project_file = module_folder / f"project_{module_name}.py"
-            if deprecated_project_file.exists():
-                message = f"DEPRECATED: Using deprecated project file: {deprecated_project_file}"
-                message += f"\nDEPRECATED: Instead use module_{module_name}.py"
-                raise ValueError(message)
+        if (names_include is None or module_name in names_include) and (
+            names_avoid is None or module_name not in names_avoid
+        ):
+            modules.append(
+                get_module_object(
+                    module_folder, module_name, library_name_has_lib_suffix, default_registers
+                )
+            )
 
     return modules
