@@ -1,6 +1,10 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 import re
 import subprocess
@@ -17,14 +21,16 @@ def get_svn_revision_information(cwd=None):
 def svn_commands_are_available(cwd=None):
     try:
         get_svn_revision(cwd)
-    except subprocess.CalledProcessError:
+    except (subprocess.CalledProcessError, FileNotFoundError):
         return False
     return True
 
 
 def check_that_svn_commands_are_available(cwd=None):
     if not svn_commands_are_available(cwd):
-        mesg = "Could not run svn. Is the command available on PATH? Is the script called from a repo?"
+        mesg = (
+            "Could not run svn. Is the command available on PATH? Is the script called from a repo?"
+        )
         raise RuntimeError(mesg)
 
 
