@@ -1,28 +1,38 @@
-# ------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
-# ------------------------------------------------------------------------------
+#
+# This file is part of the tsfpga project.
+# https://tsfpga.com
+# https://gitlab.com/tsfpga/tsfpga
+# --------------------------------------------------------------------------------------------------
 
 import re
 
 import pytest
 
-from tsfpga import TSFPGA_EXAMPLE_MODULES
+import tsfpga
 from tsfpga.system_utils import create_file, read_file
 from tsfpga.git_utils import find_git_files
 
 
 class CopyrightHeader:
 
+    separator_line_length = 100
+
     def __init__(self, file, copyright_holder, copyright_text_lines=None):
         self._file = file
         self.comment_character = self._get_comment_character()
-        self.separator_line = f"{self.comment_character} " + "-" * (79 - len(self.comment_character))
+        self.separator_line = f"{self.comment_character} " + "-" * (
+            self.separator_line_length - 1 - len(self.comment_character)
+        )
         self.expected_copyright_header = self._get_expected_copyright_header(
-            copyright_holder, copyright_text_lines)
+            copyright_holder, copyright_text_lines
+        )
 
     def check_file(self):
         """
-        Copyright comments should be correct. It should be followed by a blank line or another comment.
+        Copyright comments should be correct. It should be followed by a blank line or another
+        comment.
         """
         copyright_header_re = self.expected_copyright_header.replace("(", "\\(").replace(")", "\\)")
         regexp = re.compile(copyright_header_re + rf"($|\n|{self.comment_character})")
@@ -37,7 +47,9 @@ class CopyrightHeader:
 
     def _get_expected_copyright_header(self, copyright_holder, copyright_text_lines):
         header = f"{self.separator_line}\n"
-        header += f"{self.comment_character} Copyright (c) {copyright_holder}. All rights reserved.\n"
+        header += (
+            f"{self.comment_character} Copyright (c) {copyright_holder}. All rights reserved.\n"
+        )
         if copyright_text_lines:
             header += f"{self.comment_character}\n"
             for copyright_text_line in copyright_text_lines:
@@ -52,13 +64,18 @@ class CopyrightHeader:
             return "--"
         if self._file.name.endswith(".tcl"):
             return "#"
-        if self._file.name.endswith(".c") or self._file.name.endswith(".cpp") or self._file.name.endswith(".h"):
+        if (
+            self._file.name.endswith(".c")
+            or self._file.name.endswith(".cpp")
+            or self._file.name.endswith(".h")
+        ):
             return "//"
         raise RuntimeError(f"Could not decide file ending of {self._file}")
 
     def _is_suitable_for_insertion(self):
         """
-        If the file does not begin with a comment, we consired it suitable to insert a new copyright header comment.
+        If the file does not begin with a comment, we consired it suitable to insert a new copyright
+        header comment.
         """
         return not read_file(self._file).startswith(self.comment_character)
 
@@ -70,26 +87,37 @@ class CopyrightHeader:
 
 def files_to_check_for_copyright_header():
     file_endings = (".py", ".vhd", ".tcl")
-    exclude_directories = [TSFPGA_EXAMPLE_MODULES / "artyz7" / "tcl"]
+    exclude_directories = [tsfpga.TSFPGA_EXAMPLE_MODULES / "artyz7" / "tcl"]
     for file_ending in file_endings:
-        for file in find_git_files(file_endings_include=file_ending, exclude_directories=exclude_directories):
+        for file in find_git_files(
+            directory=tsfpga.REPO_ROOT,
+            exclude_directories=exclude_directories,
+            file_endings_include=file_ending,
+        ):
             yield file
 
 
 def test_copyright_header_of_all_checked_in_files():
     test_ok = True
     for file in files_to_check_for_copyright_header():
-        copyright_header_checker = CopyrightHeader(file, "Lukas Vik")
+        copyright_lines = [
+            "This file is part of the tsfpga project.",
+            "https://tsfpga.com",
+            "https://gitlab.com/tsfpga/tsfpga",
+        ]
+        copyright_header_checker = CopyrightHeader(file, "Lukas Vik", copyright_lines)
         if not copyright_header_checker.check_file():
             test_ok = False
-            print(f"Fail for {file}.\nExpected:\n{copyright_header_checker.expected_copyright_header}")
+            print(
+                f"Fail for {file}.\nExpected:\n{copyright_header_checker.expected_copyright_header}"
+            )
     assert test_ok
 
 
 def test_check_file(tmp_path):
-    header = "-- " + "-" * 77 + "\n"
+    header = "-- " + "-" * 97 + "\n"
     header += "-- Copyright (c) Apa. All rights reserved.\n"
-    header += "-- " + "-" * 77 + "\n"
+    header += "-- " + "-" * 97 + "\n"
 
     file = create_file(tmp_path / "header.vhd", header)
     copyright_header = CopyrightHeader(file, "Apa")
@@ -109,12 +137,12 @@ def test_check_file(tmp_path):
 
 
 def test_check_file_with_copyright_text(tmp_path):
-    header = "-- " + "-" * 77 + "\n"
+    header = "-- " + "-" * 97 + "\n"
     header += "-- Copyright (c) Apa. All rights reserved.\n"
     header += "--\n"
     header += "-- Some more\n"
     header += "-- text.\n"
-    header += "-- " + "-" * 77 + "\n"
+    header += "-- " + "-" * 97 + "\n"
 
     file = create_file(tmp_path / "header.vhd", header)
     copyright_header = CopyrightHeader(file, "Apa", ["Some more", "text."])
@@ -129,9 +157,9 @@ def test_fix_file_comment_insertion(tmp_path):
     copyright_header.fix_file()
 
     data = read_file(file).split("\n")
-    assert data[0] == "-- " + "-" * 77
+    assert data[0] == "-- " + "-" * 97
     assert data[1] == "-- Copyright (c) Hest Hestsson. All rights reserved."
-    assert data[2] == "-- " + "-" * 77
+    assert data[2] == "-- " + "-" * 97
     assert data[3] == ""
     assert data[4] == "Apa"
 
